@@ -24,158 +24,185 @@
 #include "GeoDataExtendedData.h"
 #include "GeoDataStyleMap.h"
 #include "GeoDataTypes.h"
+#include "MarbleClock.h"
 #include "MarbleDirs.h"
 #include "MarbleDebug.h"
+#include "MarbleModel.h"
+#include "MarbleRunnerManager.h"
 
 namespace Marble
 {
 
-// distance of 180deg in arcminutes
-const qreal INT2RAD = M_PI / 10800.0;
+class FileLoaderPrivate
+{
+public:
+    FileLoaderPrivate( FileLoader* parent, MarbleModel *model,
+                       const QString& file, DocumentRole role )
+        : q( parent),
+          m_runner( new MarbleRunnerManager( model->pluginManager(), q ) ),
+          m_filepath ( file ),
+          m_documentRole ( role ),
+          m_document( 0 ),
+          m_clock( model->clock() )
+    {
+        m_runner->setModel( model );
+    };
 
-FileLoader::FileLoader( QObject* parent, const QString& file, DocumentRole role = UnknownDocument )
+    FileLoaderPrivate( FileLoader* parent, MarbleModel *model,
+                       const QString& contents, const QString& file, DocumentRole role )
+        : q( parent ),
+          m_runner( new MarbleRunnerManager( model->pluginManager(), q ) ),
+          m_filepath ( file ),
+          m_contents ( contents ),
+          m_documentRole ( role ),
+          m_document( 0 ),
+          m_clock( model->clock() )
+    {
+        m_runner->setModel( model );
+    };
+
+    ~FileLoaderPrivate()
+    {
+        delete m_runner;
+    }
+
+    void importKmlFromData();
+
+    void saveFile(const QString& filename );
+    void savePlacemarks(QDataStream &out, const GeoDataContainer *container);
+
+    void createFilterProperties( GeoDataContainer *container );
+    int cityPopIdx( qint64 population ) const;
+    int spacePopIdx( qint64 population ) const;
+    int areaPopIdx( qreal area ) const;
+
+    void documentParsed( GeoDataDocument *doc, const QString& error);
+
+    FileLoader *q;
+    MarbleRunnerManager *m_runner;
+    QString m_filepath;
+    QString m_contents;
+    QString m_nonExistentLocalCacheFile;
+    DocumentRole m_documentRole;
+    GeoDataDocument *m_document;
+    QString m_error;
+
+    const MarbleClock *m_clock;
+};
+
+FileLoader::FileLoader( QObject* parent, MarbleModel *model,
+                       const QString& file, DocumentRole role = UnknownDocument )
     : QThread( parent ),
-      m_filepath( file ),
-      m_contents( QString() ),
-      m_documentRole( role ),
-      m_document( 0 )
+      d( new FileLoaderPrivate( this, model, file, role ) )
 {
 }
 
-FileLoader::FileLoader( QObject* parent, const QString& contents, const QString& file, DocumentRole role = UnknownDocument)
+FileLoader::FileLoader( QObject* parent, MarbleModel *model,
+                        const QString& contents, const QString& file, DocumentRole role = UnknownDocument)
     : QThread( parent ),
-      m_filepath( file ),
-      m_contents( contents ),
-      m_documentRole( role ),
-      m_document( 0 )
+      d( new FileLoaderPrivate( this, model, contents, file, role ) )
 {
 }
 
 FileLoader::~FileLoader()
 {
+    delete d;
 }
 
 QString FileLoader::path() const
 {
-    return m_filepath;
+    return d->m_filepath;
+}
+
+GeoDataDocument* FileLoader::document()
+{
+    return d->m_document;
+}
+
+QString FileLoader::error() const
+{
+    return d->m_error;
 }
 
 void FileLoader::run()
 {
-    if ( m_contents.isEmpty() ) {
-        QString defaultCacheName;
+    if ( d->m_contents.isEmpty() ) {
         QString defaultSourceName;
 
-        mDebug() << "starting parser for" << m_filepath;
+        mDebug() << "starting parser for" << d->m_filepath;
 
-        QFileInfo fileinfo( m_filepath );
+        QFileInfo fileinfo( d->m_filepath );
         QString path = fileinfo.path();
         if ( path == "." ) path.clear();
         QString name = fileinfo.completeBaseName();
         QString suffix = fileinfo.suffix();
+        QString cacheFile;
 
+        // determine source, cache names
         if ( fileinfo.isAbsolute() ) {
             // We got an _absolute_ path now: e.g. "/patrick.kml"
-            // defaultCacheName = path + '/' + name + ".cache";
             defaultSourceName   = path + '/' + name + '.' + suffix;
         }
         else {
-            if ( m_filepath.contains( '/' ) ) {
+            if ( d->m_filepath.contains( '/' ) ) {
                 // _relative_ path: "maps/mars/viking/patrick.kml"
-                // defaultCacheName = MarbleDirs::path( path + '/' + name + ".cache" );
                 defaultSourceName   = MarbleDirs::path( path + '/' + name + '.' + suffix );
             }
             else {
                 // _standard_ shared placemarks: "placemarks/patrick.kml"
-                defaultCacheName = MarbleDirs::path( "placemarks/" + path + name + ".cache" );
-		if (defaultCacheName.isEmpty()) {
-			defaultCacheName = MarbleDirs::localPath() + "/placemarks/" + path + name + ".cache";
+                cacheFile = MarbleDirs::path( "placemarks/" + path + name + ".cache" );
+                if ( cacheFile.isEmpty()) {
+                    cacheFile = MarbleDirs::localPath() + "/placemarks/" + path + name + ".cache";
+                    if ( !QFileInfo( cacheFile ).exists() ) {
+                        d->m_nonExistentLocalCacheFile = cacheFile;
+                    }
 		}
 		defaultSourceName   = MarbleDirs::path( "placemarks/" + path + name + '.' + suffix );
             }
         }
 
-        if ( QFile::exists( defaultCacheName ) ) {
-            mDebug() << "Loading Cache File:" + defaultCacheName;
+        // if cache file more recent that source file, load cache file
+        if ( QFile::exists( cacheFile ) ) {
+            mDebug() << "Loading Cache File:" + cacheFile;
 
-            bool      cacheoutdated = false;
             QDateTime sourceLastModified;
-            QDateTime cacheLastModified;
 
             if ( QFile::exists( defaultSourceName ) ) {
                 sourceLastModified = QFileInfo( defaultSourceName ).lastModified();
-                cacheLastModified  = QFileInfo( defaultCacheName ).lastModified();
-
-                if ( cacheLastModified < sourceLastModified )
-                    cacheoutdated = true;
             }
 
-            if ( !cacheoutdated ) {
-                loadFile( defaultCacheName );
+            const QDateTime cacheLastModified  = QFileInfo( cacheFile ).lastModified();
+
+            if ( sourceLastModified < cacheLastModified ) {
+                connect( d->m_runner, SIGNAL( parsingFinished( GeoDataDocument*, QString ) ),
+                         this, SLOT( documentParsed( GeoDataDocument*, QString ) ) );
+                d->m_runner->parseFile( cacheFile, d->m_documentRole );
             }
         }
+        // we load source file, multiple cases
         else {
-            if( suffix.compare( "pnt", Qt::CaseInsensitive ) == 0 ) {
-                loadPntFile( m_filepath );
-            }
-            else {
-                mDebug() << "No recent Default Placemark Cache File available!";
-                if ( QFile::exists( defaultSourceName ) ) {
-                    // Read the KML file.
-                    importKml( defaultSourceName );
-
-                    if (!defaultCacheName.isEmpty() ) {
-                        saveFile(defaultCacheName);
-                    }
-                }
-                else {
-                    mDebug() << "No Default Placemark Source File for " << name;
-                }
+            mDebug() << "No recent Default Placemark Cache File available!";
+            if ( QFile::exists( defaultSourceName ) ) {
+                // use runners: pnt, gpx, osm
+                connect( d->m_runner, SIGNAL( parsingFinished(GeoDataDocument*,QString) ),
+                        this, SLOT( documentParsed( GeoDataDocument*, QString ) ) );
+                d->m_runner->parseFile( defaultSourceName, d->m_documentRole );
+            } else {
+                mDebug() << "No Default Placemark Source File for " << name;
             }
         }
+    // content is not empty, we load from data
     } else {
         // Read the KML Data
-        importKmlFromData();
+        d->importKmlFromData();
+        emit loaderFinished( this );
     }
 
-    emit loaderFinished( this );
 }
 
 const quint32 MarbleMagicNumber = 0x31415926;
 
-void FileLoader::importKml( const QString& filename )
-{
-    GeoDataParser parser( GeoData_UNKNOWN );
-
-    QFile file( filename );
-    if ( !file.exists() ) {
-        qWarning( "File does not exist!" );
-        return;
-    }
-
-    // Open file in right mode
-    file.open( QIODevice::ReadOnly );
-
-    if ( !parser.read( &file ) ) {
-        qWarning( "Could not parse file!" );
-        return;
-    }
-    GeoDocument* document = parser.releaseDocument();
-    Q_ASSERT( document );
-
-    m_document = static_cast<GeoDataDocument*>( document );
-    m_document->setDocumentRole( m_documentRole );
-    m_document->setFileName( m_filepath );
-    setupStyle( m_document, m_document );
-    createFilterProperties( m_document );
-    file.close();
-
-    mDebug() << "newGeoDataDocumentAdded" << m_filepath;
-
-    emit newGeoDataDocumentAdded( m_document );
-}
-
-void FileLoader::importKmlFromData()
+void FileLoaderPrivate::importKmlFromData()
 {
     GeoDataParser parser( GeoData_KML );
 
@@ -184,7 +211,7 @@ void FileLoader::importKmlFromData()
     buffer.open( QIODevice::ReadOnly );
 
     if ( !parser.read( &buffer ) ) {
-        qWarning( "Could not parse buffer!" );
+        qWarning( "Could not import kml buffer!" );
         return;
     }
     GeoDocument* document = parser.releaseDocument();
@@ -193,93 +220,15 @@ void FileLoader::importKmlFromData()
     m_document = static_cast<GeoDataDocument*>( document );
     m_document->setDocumentRole( m_documentRole );
     m_document->setFileName( m_filepath );
-    setupStyle( m_document, m_document );
-
+    createFilterProperties( m_document );
     buffer.close();
 
     mDebug() << "newGeoDataDocumentAdded" << m_filepath;
 
-    emit newGeoDataDocumentAdded( m_document );
+    emit q->newGeoDataDocumentAdded( m_document );
 }
 
-void FileLoader::loadFile( const QString &filename )
-{
-    QFile file( filename );
-    file.open( QIODevice::ReadOnly );
-    QDataStream in( &file );
-
-    // Read and check the header
-    quint32 magic;
-    in >> magic;
-    if ( magic != MarbleMagicNumber ) {
-        qDebug( "Bad file format!" );
-        return;
-    }
-
-    // Read the version
-    qint32 version;
-    in >> version;
-    if ( version < 015 ) {
-        qDebug( "Bad file - too old!" );
-        return;
-    }
-    /*
-      if (version > 002) {
-      qDebug( "Bad file - too new!" );
-      return;
-      }
-    */
-    m_document = new GeoDataDocument();
-    m_document->setDocumentRole( m_documentRole );
-    m_document->setFileName( m_filepath );
-
-    in.setVersion( QDataStream::Qt_4_2 );
-
-    // Read the data itself
-    // Use double to provide a single cache file format across architectures
-    double   lon;
-    double   lat;
-    double   alt;
-    double   area;
-
-    QString  tmpstr;
-    qint64   tmpint64;
-    qint8    tmpint8;
-    qint16   tmpint16;
-
-    while ( !in.atEnd() ) {
-        GeoDataPlacemark *mark = new GeoDataPlacemark;
-        in >> tmpstr;
-        mark->setName( tmpstr );
-        in >> lon >> lat >> alt;
-        mark->setCoordinate( (qreal)(lon), (qreal)(lat), (qreal)(alt) );
-        in >> tmpstr;
-        mark->setRole( tmpstr );
-        in >> tmpstr;
-        mark->setDescription( tmpstr );
-        in >> tmpstr;
-        mark->setCountryCode( tmpstr );
-        in >> tmpstr;
-        mark->setState( tmpstr );
-        in >> area;
-        mark->setArea( (qreal)(area) );
-        in >> tmpint64;
-        mark->setPopulation( tmpint64 );
-        in >> tmpint16;
-        mark->extendedData().addValue( GeoDataData( "gmt", int( tmpint16 ) ) );
-        in >> tmpint8;
-        mark->extendedData().addValue( GeoDataData( "dst", int( tmpint8 ) ) );
-
-        m_document->append( mark );
-    }
-
-    m_document->setVisible( false );
-    setupStyle( m_document, m_document );
-    createFilterProperties( m_document );
-    emit newGeoDataDocumentAdded( m_document );
-}
-
-void FileLoader::saveFile( const QString& filename )
+void FileLoaderPrivate::saveFile( const QString& filename )
 {
 
     if ( !QDir( MarbleDirs::localPath() + "/placemarks/" ).exists() )
@@ -288,7 +237,10 @@ void FileLoader::saveFile( const QString& filename )
     mDebug() << "Creating cache at " << filename ;
 
     QFile file( filename );
-    file.open( QIODevice::WriteOnly );
+    if ( !file.open( QIODevice::WriteOnly ) ) {
+        mDebug() << Q_FUNC_INFO << "Can't open" << filename << "for writing";
+        return;
+    }
     QDataStream out( &file );
 
     // Write a header with a "magic number" and a version
@@ -301,7 +253,7 @@ void FileLoader::saveFile( const QString& filename )
     savePlacemarks(out, m_document);
 }
 
-void FileLoader::savePlacemarks(QDataStream &out, const GeoDataContainer *container)
+void FileLoaderPrivate::savePlacemarks(QDataStream &out, const GeoDataContainer *container)
 {
 
     qreal lon;
@@ -313,7 +265,7 @@ void FileLoader::savePlacemarks(QDataStream &out, const GeoDataContainer *contai
     QVector<GeoDataPlacemark*>::const_iterator const end = placemarks.constEnd();
     for (; it != end; ++it ) {
         out << (*it)->name();
-        (*it)->coordinate( lon, lat, alt );
+        (*it)->coordinate( m_clock->dateTime() ).geoCoordinates( lon, lat, alt );
 
         // Use double to provide a single cache file format across architectures
         out << (double)(lon) << (double)(lat) << (double)(alt);
@@ -335,88 +287,23 @@ void FileLoader::savePlacemarks(QDataStream &out, const GeoDataContainer *contai
     }
 }
 
-void FileLoader::loadPntFile( const QString &fileName )
+void FileLoaderPrivate::documentParsed( GeoDataDocument* doc, const QString& error )
 {
-    QFile  file( fileName );
-
-    file.open( QIODevice::ReadOnly );
-    QDataStream stream( &file );  // read the data serialized from the file
-    stream.setByteOrder( QDataStream::LittleEndian );
-
-    m_document = new GeoDataDocument();
-    m_document->setDocumentRole( m_documentRole );
-    m_document->setFileName( fileName );
-
-    short  header;
-    short  iLat;
-    short  iLon;
-
-    GeoDataPlacemark  *placemark = 0;
-    placemark = new GeoDataPlacemark;
-    m_document->append( placemark );
-    GeoDataMultiGeometry *geom = new GeoDataMultiGeometry;
-    placemark->setGeometry( geom );
-
-    while( !stream.atEnd() ){
-        stream >> header >> iLat >> iLon;
-        // Transforming Range of Coordinates to iLat [0,ARCMINUTE] , iLon [0,2 * ARCMINUTE]
-
-        if ( header > 5 ) {
-
-            // qDebug(QString("header: %1 iLat: %2 iLon: %3").arg(header).arg(iLat).arg(iLon).toLatin1());
-
-            // Find out whether the Polyline is a river or a closed polygon
-            if ( ( header >= 7000 && header < 8000 )
-                || ( header >= 9000 && header < 20000 ) ) {
-                GeoDataLineString *polyline = new GeoDataLineString;
-                geom->append( polyline );
-            }
-            else {
-                GeoDataLinearRing *polyline = new GeoDataLinearRing;
-                geom->append( polyline );
-            }
-        }
-        GeoDataLineString *polyline = static_cast<GeoDataLineString*>(geom->child(geom->size()-1));
-        polyline->append( GeoDataCoordinates( (qreal)(iLon) * INT2RAD, (qreal)(iLat) * INT2RAD,
-                                                  0.0, GeoDataCoordinates::Radian, 5 ) );
-    }
-
-    file.close();
-
-    emit newGeoDataDocumentAdded( m_document );
-
-}
-
-void FileLoader::setupStyle( GeoDataDocument *doc, GeoDataContainer *container )
-{
-    QVector<GeoDataFeature*>::Iterator i = container->begin();
-    QVector<GeoDataFeature*>::Iterator const end = container->end();
-    for (; i != end; ++i ) {
-        if ( (*i)->nodeType() == GeoDataTypes::GeoDataFolderType
-             || (*i)->nodeType() == GeoDataTypes::GeoDataDocumentType ) {
-            GeoDataContainer *child = static_cast<GeoDataContainer*>( *i );
-            setupStyle( doc, child );
-        } else {
-            GeoDataPlacemark* placemark = static_cast<GeoDataPlacemark*>( *i );
-
-            QString styleUrl = placemark->styleUrl().remove('#');
-            if ( ! styleUrl.isEmpty() )
-            {
-                const GeoDataStyleMap& styleMap = doc->styleMap( styleUrl );
-                /// hard coded to use only the "normal" style
-                if( !styleMap.value( QString( "normal" ) ).isEmpty() ) {
-                    styleUrl = styleMap.value( QString( "normal" ) );
-                }
-                styleUrl.remove( '#' );
-                placemark->setStyle( &doc->style( styleUrl ) );
-            }
+    m_error = error;
+    if ( doc ) {
+        m_document = doc;
+        doc->setFileName( m_filepath );
+        createFilterProperties( doc );
+        emit q->newGeoDataDocumentAdded( m_document );
+        if ( !m_nonExistentLocalCacheFile.isEmpty() ) {
+            saveFile( m_nonExistentLocalCacheFile );
         }
     }
+    emit q->loaderFinished( q );
 }
 
-void FileLoader::createFilterProperties( GeoDataContainer *container )
+void FileLoaderPrivate::createFilterProperties( GeoDataContainer *container )
 {
-
     QVector<GeoDataFeature*>::Iterator i = container->begin();
     QVector<GeoDataFeature*>::Iterator const end = container->end();
     for (; i != end; ++i ) {
@@ -432,7 +319,7 @@ void FileLoader::createFilterProperties( GeoDataContainer *container )
             // Mountain (H), Volcano (V), Shipwreck (W)
             if ( placemark->role() == "H" || placemark->role() == "V" || placemark->role() == "W" )
             {
-                qreal altitude = placemark->coordinate().altitude();
+                qreal altitude = placemark->coordinate( m_clock->dateTime() ).altitude();
                 if ( altitude != 0.0 )
                 {
                     hasPopularity = true;
@@ -553,17 +440,17 @@ void FileLoader::createFilterProperties( GeoDataContainer *container )
                    || placemark->role()=="PPLR"
                    || placemark->role()=="PPLS"
                    || placemark->role()=="PPLW" ) placemark->setVisualCategory(
-                        ( ( GeoDataPlacemark::GeoDataVisualCategory )( (int)( GeoDataPlacemark::SmallCity )
-                                                                       + ( placemark->popularityIndex() -1 ) / 4 * 4 ) ) );
+                        ( GeoDataPlacemark::GeoDataVisualCategory )( GeoDataPlacemark::SmallCity
+                                                                       + ( placemark->popularityIndex() -1 ) / 4 * 4 ) );
             else if ( placemark->role() == "PPLA" ) placemark->setVisualCategory(
-                    ( ( GeoDataPlacemark::GeoDataVisualCategory )( (int)( GeoDataPlacemark::SmallStateCapital )
-                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) ) );
+                    ( GeoDataPlacemark::GeoDataVisualCategory )( GeoDataPlacemark::SmallStateCapital
+                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) );
             else if ( placemark->role()=="PPLC" ) placemark->setVisualCategory(
-                    ( ( GeoDataPlacemark::GeoDataVisualCategory )( (int)( GeoDataPlacemark::SmallNationCapital )
-                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) ) );
+                    ( GeoDataPlacemark::GeoDataVisualCategory )( GeoDataPlacemark::SmallNationCapital
+                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) );
             else if ( placemark->role()=="PPLA2" || placemark->role()=="PPLA3" ) placemark->setVisualCategory(
-                    ( ( GeoDataPlacemark::GeoDataVisualCategory )( (int)( GeoDataPlacemark::SmallCountyCapital )
-                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) ) );
+                    ( GeoDataPlacemark::GeoDataVisualCategory )( GeoDataPlacemark::SmallCountyCapital
+                                                                   + ( placemark->popularityIndex() -1 ) / 4 * 4 ) );
             else if ( placemark->role()==" " && !hasPopularity && placemark->visualCategory() == GeoDataPlacemark::Unknown ) {
                 placemark->setVisualCategory( GeoDataPlacemark::Unknown ); // default location
                 placemark->setPopularityIndex(0);
@@ -591,7 +478,7 @@ void FileLoader::createFilterProperties( GeoDataContainer *container )
     }
 }
 
-int FileLoader::cityPopIdx( qint64 population ) const
+int FileLoaderPrivate::cityPopIdx( qint64 population ) const
 {
     int popidx = 15;
 
@@ -613,7 +500,7 @@ int FileLoader::cityPopIdx( qint64 population ) const
     return popidx;
 }
 
-int FileLoader::spacePopIdx( qint64 population ) const
+int FileLoaderPrivate::spacePopIdx( qint64 population ) const
 {
     int popidx = 18;
 
@@ -636,7 +523,7 @@ int FileLoader::spacePopIdx( qint64 population ) const
     return popidx;
 }
 
-int FileLoader::areaPopIdx( qreal area ) const
+int FileLoaderPrivate::areaPopIdx( qreal area ) const
 {
     int popidx = 17;
     if      ( area <  200000  )      popidx=11;
