@@ -101,11 +101,10 @@ GeoDataCoordinates GeoDataLineStringPrivate::findDateLine( const GeoDataCoordina
 
     qreal  lon = 0.0;
     qreal  lat = 0.0;
-    Quaternion  itpos;
 
     qreal altDiff = currentCoords.altitude() - previousCoords.altitude();
 
-    itpos.nlerp( previousCoords.quaternion(), currentCoords.quaternion(), 0.5 );
+    const Quaternion itpos = Quaternion::nlerp( previousCoords.quaternion(), currentCoords.quaternion(), 0.5 );
     itpos.getSpherical( lon, lat );
 
     qreal altitude = previousCoords.altitude() + 0.5 * altDiff;
@@ -230,7 +229,7 @@ GeoDataLineString& GeoDataLineString::operator << ( const GeoDataLineString& val
 {
     GeoDataGeometry::detach();
     GeoDataLineStringPrivate* d = p();
-    qDeleteAll( d->m_rangeCorrected );
+    //qDeleteAll( d->m_rangeCorrected );
     d->m_rangeCorrected.clear();
     d->m_dirtyRange = true;
     d->m_dirtyBox = true;
@@ -249,7 +248,7 @@ void GeoDataLineString::clear()
 {
     GeoDataGeometry::detach();
     GeoDataLineStringPrivate* d = p();
-    qDeleteAll( d->m_rangeCorrected );
+    //qDeleteAll( d->m_rangeCorrected );
     d->m_rangeCorrected.clear();
     d->m_dirtyRange = true;
     d->m_dirtyBox = true;
