@@ -14,6 +14,8 @@
 #include "GeoWriter.h"
 #include "KmlElementDictionary.h"
 #include "MarbleDebug.h"
+#include "GeoDataExtendedData.h"
+#include "GeoDataTimeStamp.h"
 
 #include "GeoDataTypes.h"
 
@@ -35,6 +37,10 @@ bool KmlFolderTagWriter::write( const GeoNode *node, GeoWriter& writer ) const
     //Writing folder name
     writer.writeOptionalElement( "name", folder->name() );
 
+    if( !folder->extendedData().isEmpty() ){
+        writeElement( &folder->extendedData(), writer );
+    }
+
     //Write all containing features
     QVector<GeoDataFeature*>::ConstIterator it =  folder->constBegin();
     QVector<GeoDataFeature*>::ConstIterator const end = folder->constEnd();
@@ -43,6 +49,9 @@ bool KmlFolderTagWriter::write( const GeoNode *node, GeoWriter& writer ) const
         writeElement( *it, writer );
     }
 
+    if( folder->timeStamp().when().isValid() )
+        writeElement( &folder->timeStamp(), writer );
+
     //close folder tag
     writer.writeEndElement();
     return true;
